@@ -20,7 +20,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id$
+ * $Id: xvid_decraw.c,v 1.22 2005/10/07 15:02:28 suxen_drol Exp $
  *
  ****************************************************************************/
 
@@ -54,9 +54,6 @@
  *               Global vars in module and constants
  ****************************************************************************/
 
-/* max number of frames */
-#define ABS_MAXFRAMENR 9999
-
 #define USE_PNM 0
 #define USE_TGA 1
 
@@ -74,12 +71,16 @@ static void *dec_handle = NULL;
 
 #define BUFFER_SIZE (2*1024*1024)
 
+static const int display_buffer_bytes = 0;
+
+#define MIN_USEFUL_BYTES 1
+
 /*****************************************************************************
  *               Local prototypes
  ****************************************************************************/
 
 static double msecond();
-static int dec_init(int use_assembler);
+static int dec_init(int use_assembler, int debug_level);
 static int dec_main(unsigned char *istream,
 					unsigned char *ostream,
 					int istream_size,
@@ -111,6 +112,7 @@ int main(int argc, char *argv[])
 	unsigned char *mp4_ptr    = NULL;
 	unsigned char *out_buffer = NULL;
 	int useful_bytes;
+	int chunk;
 	xvid_dec_stats_t xvid_dec_stats;
 	
 	double totaldectime;
@@ -119,6 +121,7 @@ int main(int argc, char *argv[])
 	int status;
   
 	int use_assembler = 0;
+	int debug_level = 0;
   
 	char filename[256];
   
@@ -137,6 +140,11 @@ int main(int argc, char *argv[])
  
 		if (strcmp("-asm", argv[i]) == 0 ) {
 			use_assembler = 1;
+		} else if (strcmp("-debug", argv[i]) == 0 && i < argc - 1 ) {
+			i++;
+			if (sscanf(argv[i], "0x%x", &debug_level) != 1) {
+				debug_level = atoi(argv[i]);
+			}
 		} else if (strcmp("-d", argv[i]) == 0) {
 			ARG_SAVEDECOUTPUT = 1;
 		} else if (strcmp("-i", argv[i]) == 0 && i < argc - 1 ) {
@@ -219,7 +227,7 @@ int main(int argc, char *argv[])
  *        XviD PART  Start
  ****************************************************************************/
 
-	status = dec_init(use_assembler);
+	status = dec_init(use_assembler, debug_level);
 	if (status) {
 		fprintf(stderr,
 				"Decore INIT problem, return value %d\n", status);
@@ -238,7 +246,8 @@ int main(int argc, char *argv[])
 	totalsize = 0;
 	filenr = 0;
 	mp4_ptr = mp4_buffer;
-
+	chunk = 0;
+	
 	do {
 		int used_bytes = 0;
 		double dectime;
@@ -296,6 +305,22 @@ int main(int argc, char *argv[])
 
 					fprintf(stderr, "Resized frame buffer to %dx%d\n", XDIM, YDIM);
 				}
+
+				/* Save individual mpeg4 stream if required */
+				if(ARG_SAVEMPEGSTREAM) {
+					FILE *filehandle = NULL;
+
+					sprintf(filename, "%svolhdr.m4v", filepath);
+					filehandle = fopen(filename, "wb");
+					if(!filehandle) {
+						fprintf(stderr,
+								"Error writing vol header mpeg4 stream to file %s\n",
+								filename);
+					} else {
+						fwrite(mp4_ptr, 1, used_bytes, filehandle);
+						fclose(filehandle);
+					}
+				}
 			}
 
 			/* Update buffer pointers */
@@ -307,7 +332,10 @@ int main(int argc, char *argv[])
 				totalsize += used_bytes;
 			}
 
-		}while(xvid_dec_stats.type <= 0 && useful_bytes > 0);
+			if (display_buffer_bytes) {
+				printf("Data chunk %d: %d bytes consumed, %d bytes in buffer\n", chunk++, used_bytes, useful_bytes);
+			}
+		} while (xvid_dec_stats.type <= 0 && useful_bytes > MIN_USEFUL_BYTES);
 
 		/* Check if there is a negative number of useful bytes left in buffer
 		 * This means we went too far */
@@ -318,9 +346,11 @@ int main(int argc, char *argv[])
 		totaldectime += dectime;
 
 			
-        printf("Frame %5d: type = %s, dectime(ms) =%6.1f, length(bytes) =%7d\n",
-			   filenr, type2str(xvid_dec_stats.type), dectime, used_bytes);
-			
+		if (!display_buffer_bytes) {
+			printf("Frame %5d: type = %s, dectime(ms) =%6.1f, length(bytes) =%7d\n",
+					filenr, type2str(xvid_dec_stats.type), dectime, used_bytes);
+		}
+
 		/* Save individual mpeg4 stream if required */
 		if(ARG_SAVEMPEGSTREAM) {
 			FILE *filehandle = NULL;
@@ -350,7 +380,9 @@ int main(int argc, char *argv[])
 
 		filenr++;
 
-	} while ( (status>=0) && (filenr<ABS_MAXFRAMENR));
+	} while (useful_bytes>MIN_USEFUL_BYTES || !feof(in_file));
+
+	useful_bytes = 0; /* Empty buffer */
 
 /*****************************************************************************
  *     Flush decoder buffers
@@ -366,7 +398,10 @@ int main(int argc, char *argv[])
 		    dectime = msecond();
 		    used_bytes = dec_main(NULL, out_buffer, -1, &xvid_dec_stats);
 		    dectime = msecond() - dectime;
-        }while(used_bytes>=0 && xvid_dec_stats.type <= 0);
+			if (display_buffer_bytes) {
+				printf("Data chunk %d: %d bytes consumed, %d bytes in buffer\n", chunk++, used_bytes, useful_bytes);
+			}
+        } while(used_bytes>=0 && xvid_dec_stats.type <= 0);
 
         if (used_bytes < 0) {   /* XVID_ERR_END */
             break;
@@ -376,9 +411,11 @@ int main(int argc, char *argv[])
 		totaldectime += dectime;
 
 		/* Prints some decoding stats */
-        printf("Frame %5d: type = %s, dectime(ms) =%6.1f, length(bytes) =%7d\n",
-			   filenr, type2str(xvid_dec_stats.type), dectime, used_bytes);
-			
+		if (!display_buffer_bytes) {
+			printf("Frame %5d: type = %s, dectime(ms) =%6.1f, length(bytes) =%7d\n",
+					filenr, type2str(xvid_dec_stats.type), dectime, used_bytes);
+		}
+
 		/* Save output frame if required */
 		if (ARG_SAVEDECOUTPUT) {
 			sprintf(filename, "%sdec%05d", filepath, filenr);
@@ -434,6 +471,7 @@ static void usage()
 	fprintf(stderr, "Usage : xvid_decraw [OPTIONS]\n");
 	fprintf(stderr, "Options :\n");
 	fprintf(stderr, " -asm           : use assembly optimizations (default=disabled)\n");
+	fprintf(stderr, " -debug         : debug level (debug=0)\n");
 	fprintf(stderr, " -i string      : input filename (default=stdin)\n");
 	fprintf(stderr, " -d             : save decoder output\n");
 	fprintf(stderr, " -c csp         : choose colorspace output (rgb16, rgb24, rgb32, yv12, i420)\n");
@@ -459,7 +497,7 @@ msecond()
 #else
 	clock_t clk;
 	clk = clock();
-	return(clk * 1000 / CLOCKS_PER_SEC);
+	return(clk * 1000.0 / CLOCKS_PER_SEC);
 #endif
 }
 
@@ -538,21 +576,21 @@ static int write_tga(char *filename, unsigned char *image)
 #else
 	{
 		int i;
-		for (i=0; i<width*height*BPP;i+=BPP) {
+		for (i=0; i<XDIM*YDIM*BPP;i+=BPP) {
 			if (BPP == 1) {
-				fputc(image+i, f);
+				fputc(*(image+i), f);
 			} else if (BPP == 2) {
-				fputc(image+i+1, f);
-				fputc(image+i+0, f);
+				fputc(*(image+i+1), f);
+				fputc(*(image+i+0), f);
 			} else if (BPP == 3) {
-				fputc(image+i+2, f);
-				fputc(image+i+1, f);
-				fputc(image+i+0, f);
+				fputc(*(image+i+2), f);
+				fputc(*(image+i+1), f);
+				fputc(*(image+i+0), f);
 			} else if (BPP == 4) {
-				fputc(image+i+3, f);
-				fputc(image+i+2, f);
-				fputc(image+i+1, f);
-				fputc(image+i+0, f);
+				fputc(*(image+i+3), f);
+				fputc(*(image+i+2), f);
+				fputc(*(image+i+1), f);
+				fputc(*(image+i+0), f);
 			}
 		}
 	}
@@ -622,7 +660,7 @@ static int write_pnm(char *filename, unsigned char *image)
 
 /* init decoder before first run */
 static int
-dec_init(int use_assembler)
+dec_init(int use_assembler, int debug_level)
 {
 	int ret;
 
@@ -650,6 +688,8 @@ dec_init(int use_assembler)
 	else
 		xvid_gbl_init.cpu_flags = XVID_CPU_FORCE;
 
+	xvid_gbl_init.debug = debug_level;
+
 	xvid_global(NULL, 0, &xvid_gbl_init, NULL);
 
 	/*------------------------------------------------------------------------
@@ -684,7 +724,10 @@ dec_main(unsigned char *istream,
 	int ret;
 
 	xvid_dec_frame_t xvid_dec_frame;
+
+	/* Reset all structures */
 	memset(&xvid_dec_frame, 0, sizeof(xvid_dec_frame_t));
+	memset(xvid_dec_stats, 0, sizeof(xvid_dec_stats_t));
 
 	/* Set version */
 	xvid_dec_frame.version = XVID_VERSION;
