@@ -19,7 +19,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id$
+ * $Id: mbcoding.c,v 1.52 2005/09/13 12:12:15 suxen_drol Exp $
  *
  ****************************************************************************/
 
@@ -38,6 +38,8 @@
 
 #define LEVELOFFSET 32
 
+/* Initialized once during xvid_global call
+ * RO access is thread safe */
 static REVERSE_EVENT DCT3D[2][4096];
 static VLC coeff_VLC[2][2][64][64];
 
@@ -197,8 +199,7 @@ init_vlc_tables(void)
 static __inline void
 CodeVector(Bitstream * bs,
 		   int32_t value,
-		   int32_t f_code,
-		   Statistics * pStat)
+		   int32_t f_code)
 {
 
 	const int scale_factor = 1 << (f_code - 1);
@@ -210,9 +211,6 @@ CodeVector(Bitstream * bs,
 	if (value > (cmp - 1))
 		value -= 64 * scale_factor;
 
-	pStat->iMvSum += value * value;
-	pStat->iMvCount++;
-
 	if (value == 0) {
 		BitstreamPutBits(bs, mb_motion_table[32].code,
 						 mb_motion_table[32].len);
@@ -453,7 +451,7 @@ CodeCoeffInter_CalcBits(const int16_t qcoeff[64], const uint16_t * zigzag)
 	return bits;
 }
 
-static int iDQtab[5] = {
+static const int iDQtab[5] = {
 	1, 0, -1 /* no change */, 2, 3
 };
 #define DQ_VALUE2INDEX(value)  iDQtab[(value)+2]
@@ -577,8 +575,8 @@ CodeBlockInter(const FRAMEINFO * const frame,
 	/* code motion vector(s) if motion is local  */
 	if (!pMB->mcsel)
 		for (i = 0; i < (pMB->mode == MODE_INTER4V ? 4 : 1); i++) {
-			CodeVector(bs, pMB->pmvs[i].x, frame->fcode, pStat);
-			CodeVector(bs, pMB->pmvs[i].y, frame->fcode, pStat);
+			CodeVector(bs, pMB->pmvs[i].x, frame->fcode);
+			CodeVector(bs, pMB->pmvs[i].y, frame->fcode);
 		}
 
 	bits = BitstreamPos(bs);
@@ -608,6 +606,12 @@ MBCoding(const FRAMEINFO * const frame,
 	if (frame->coding_type != I_VOP)
 			BitstreamPutBit(bs, 0);	/* not_coded */
 
+	if (frame->vop_flags & XVID_VOP_GREYSCALE) {
+		pMB->cbp &= 0x3C;		/* keep only bits 5-2 */
+		qcoeff[4*64+0]=0;		/* for INTRA DC value is saved */
+		qcoeff[5*64+0]=0;
+	}
+
 	if (pMB->mode == MODE_INTRA || pMB->mode == MODE_INTRA_Q)
 		CodeBlockIntra(frame, pMB, qcoeff, bs, pStat);
 	else
@@ -694,7 +698,6 @@ MBCodingBVOP(const FRAMEINFO * const frame,
 		scan_tables[2] : scan_tables[0];
 	int bits;
 
-
 /*	------------------------------------------------------------------
 		when a block is skipped it is decoded DIRECT(0,0)
 		hence is interpolated from forward & backward frames
@@ -746,17 +749,17 @@ MBCodingBVOP(const FRAMEINFO * const frame,
 
 	switch (mb->mode) {
 		case MODE_INTERPOLATE:
-			CodeVector(bs, mb->pmvs[1].x, vcode, pStat); /* forward vector of interpolate mode */
-			CodeVector(bs, mb->pmvs[1].y, vcode, pStat);
+			CodeVector(bs, mb->pmvs[1].x, vcode); /* forward vector of interpolate mode */
+			CodeVector(bs, mb->pmvs[1].y, vcode);
 		case MODE_BACKWARD:
 			vcode = bcode;
 		case MODE_FORWARD:
-			CodeVector(bs, mb->pmvs[0].x, vcode, pStat);
-			CodeVector(bs, mb->pmvs[0].y, vcode, pStat);
+			CodeVector(bs, mb->pmvs[0].x, vcode);
+			CodeVector(bs, mb->pmvs[0].y, vcode);
 			break;
 		case MODE_DIRECT:
-			CodeVector(bs, mb->pmvs[3].x, 1, pStat);	/* fcode is always 1 for delta vector */
-			CodeVector(bs, mb->pmvs[3].y, 1, pStat);	/* prediction is always (0,0) */
+			CodeVector(bs, mb->pmvs[3].x, 1);	/* fcode is always 1 for delta vector */
+			CodeVector(bs, mb->pmvs[3].y, 1);	/* prediction is always (0,0) */
 		default: break;
 	}
 
@@ -958,6 +961,8 @@ get_dc_size_chrom(Bitstream * bs)
 
 }
 
+#define GET_BITS(cache, n) ((cache)>>(32-(n)))
+
 static __inline int
 get_coeff(Bitstream * bs,
 		  int *run,
@@ -970,11 +975,13 @@ get_coeff(Bitstream * bs,
 	int32_t level;
 	REVERSE_EVENT *reverse_event;
 
+	uint32_t cache = BitstreamShowBits(bs, 32);
+	
 	if (short_video_header)		/* inter-VLCs will be used for both intra and inter blocks */
 		intra = 0;
 
-	if (BitstreamShowBits(bs, 7) != ESCAPE) {
-		reverse_event = &DCT3D[intra][BitstreamShowBits(bs, 12)];
+	if (GET_BITS(cache, 7) != ESCAPE) {
+		reverse_event = &DCT3D[intra][GET_BITS(cache, 12)];
 
 		if ((level = reverse_event->event.level) == 0)
 			goto error;
@@ -982,31 +989,35 @@ get_coeff(Bitstream * bs,
 		*last = reverse_event->event.last;
 		*run  = reverse_event->event.run;
 
-		BitstreamSkip(bs, reverse_event->len);
+		/* Don't forget to update the bitstream position */
+		BitstreamSkip(bs, reverse_event->len+1);
 
-		return BitstreamGetBits(bs, 1) ? -level : level;
+		return (GET_BITS(cache, reverse_event->len+1)&0x01) ? -level : level;
 	}
 
-	BitstreamSkip(bs, 7);
+	/* flush 7bits of cache */
+	cache <<= 7;
 
 	if (short_video_header) {
 		/* escape mode 4 - H.263 type, only used if short_video_header = 1  */
-		*last = BitstreamGetBit(bs);
-		*run = BitstreamGetBits(bs, 6);
-		level = BitstreamGetBits(bs, 8);
+		*last =  GET_BITS(cache, 1);
+		*run  = (GET_BITS(cache, 7) &0x3f);
+		level = (GET_BITS(cache, 15)&0xff);
 
 		if (level == 0 || level == 128)
 			DPRINTF(XVID_DEBUG_ERROR, "Illegal LEVEL for ESCAPE mode 4: %d\n", level);
 
+		/* We've "eaten" 22 bits */
+		BitstreamSkip(bs, 22);
+
 		return (level << 24) >> 24;
 	}
 
-	mode = BitstreamShowBits(bs, 2);
+	if ((mode = GET_BITS(cache, 2)) < 3) {
+		const int skip[3] = {1, 1, 2};
+		cache <<= skip[mode];
 
-	if (mode < 3) {
-		BitstreamSkip(bs, (mode == 2) ? 2 : 1);
-
-		reverse_event = &DCT3D[intra][BitstreamShowBits(bs, 12)];
+		reverse_event = &DCT3D[intra][GET_BITS(cache, 12)];
 
 		if ((level = reverse_event->event.level) == 0)
 			goto error;
@@ -1014,23 +1025,28 @@ get_coeff(Bitstream * bs,
 		*last = reverse_event->event.last;
 		*run  = reverse_event->event.run;
 
-		BitstreamSkip(bs, reverse_event->len);
-
-		if (mode < 2)			/* first escape mode, level is offset */
+		if (mode < 2) {
+			/* first escape mode, level is offset */
 			level += max_level[intra][*last][*run];
-		else					/* second escape mode, run is offset */
+		} else {
+			/* second escape mode, run is offset */
 			*run += max_run[intra][*last][level] + 1;
+		}
+		
+		/* Update bitstream position */
+		BitstreamSkip(bs, 7 + skip[mode] + reverse_event->len + 1);
 
-		return BitstreamGetBits(bs, 1) ? -level : level;
+		return (GET_BITS(cache, reverse_event->len+1)&0x01) ? -level : level;
 	}
 
 	/* third escape mode - fixed length codes */
-	BitstreamSkip(bs, 2);
-	*last = BitstreamGetBits(bs, 1);
-	*run = BitstreamGetBits(bs, 6);
-	BitstreamSkip(bs, 1);		/* marker */
-	level = BitstreamGetBits(bs, 12);
-	BitstreamSkip(bs, 1);		/* marker */
+	cache <<= 2;
+	*last =  GET_BITS(cache, 1);
+	*run  = (GET_BITS(cache, 7)&0x3f);
+	level = (GET_BITS(cache, 20)&0xfff);
+	
+	/* Update bitstream position */
+	BitstreamSkip(bs, 30);
 
 	return (level << 20) >> 20;
 
@@ -1056,6 +1072,14 @@ get_intra_block(Bitstream * bs,
 			break;
 		}
 		coeff += run;
+		
+#ifdef _DEBUG
+		if(coeff>=64) {
+		  DPRINTF(XVID_DEBUG_ERROR,"error: overflow in coefficient index\n");
+		  return;
+		}
+#endif
+
 		block[scan[coeff]] = level;
 
 		DPRINTF(XVID_DEBUG_COEFF,"block[%i] %i\n", scan[coeff], level);
@@ -1072,12 +1096,17 @@ get_intra_block(Bitstream * bs,
 }
 
 void
-get_inter_block(Bitstream * bs,
-				int16_t * block,
-				int direction)
+get_inter_block_h263(
+		Bitstream * bs,
+		int16_t * block,
+		int direction,
+		const int quant,
+		const uint16_t *matrix)
 {
 
 	const uint16_t *scan = scan_tables[direction];
+	const uint16_t quant_m_2 = quant << 1;
+	const uint16_t quant_add = (quant & 1 ? quant : quant - 1);
 	int p;
 	int level;
 	int run;
@@ -1092,17 +1121,72 @@ get_inter_block(Bitstream * bs,
 		}
 		p += run;
 
-		block[scan[p]] = level;
-
-		DPRINTF(XVID_DEBUG_COEFF,"block[%i] %i\n", scan[p], level);
-		/* DPRINTF(XVID_DEBUG_COEFF,"block[%i] %i %08x\n", scan[p], level, BitstreamShowBits(bs, 32)); */
-
-		if (level < -2047 || level > 2047) {
-			DPRINTF(XVID_DEBUG_ERROR,"warning: inter overflow %i\n", level);
+#ifdef _DEBUG
+		if(p>=64)	{
+		  DPRINTF(XVID_DEBUG_ERROR,"error: overflow in coefficient index\n");
+		  return;
 		}
+#endif
+
+		if (level < 0) {
+			level = level*quant_m_2 - quant_add;
+			block[scan[p]] = (level >= -2048 ? level : -2048);
+		} else {
+			level = level * quant_m_2 + quant_add;
+			block[scan[p]] = (level <= 2047 ? level : 2047);
+		}		
+		p++;
+	} while (!last);
+}
+
+void
+get_inter_block_mpeg(
+		Bitstream * bs,
+		int16_t * block,
+		int direction,
+		const int quant,
+		const uint16_t *matrix)
+{
+	const uint16_t *scan = scan_tables[direction];
+	uint32_t sum = 0;
+	int p;
+	int level;
+	int run;
+	int last;
+
+	p = 0;
+	do {
+		level = get_coeff(bs, &run, &last, 0, 0);
+		if (run == -1) {
+			DPRINTF(XVID_DEBUG_ERROR,"fatal: invalid run");
+			break;
+		}
+		p += run;
+
+#ifdef _DEBUG
+		if(p>=64)	{
+		  DPRINTF(XVID_DEBUG_ERROR,"error: overflow in coefficient index\n");
+		  return;
+		}
+#endif
+
+		if (level < 0) {
+			level = ((2 * -level + 1) * matrix[scan[p]] * quant) >> 4;
+			block[scan[p]] = (level <= 2048 ? -level : -2048);
+		} else {
+			level = ((2 *  level + 1) * matrix[scan[p]] * quant) >> 4;
+			block[scan[p]] = (level <= 2047 ? level : 2047);
+		}
+
+		sum ^= block[scan[p]];
+		
 		p++;
 	} while (!last);
 
+	/*	mismatch control */
+	if ((sum & 1) == 0) {
+		block[63] ^= 1;
+	}
 }
 
 
@@ -1882,14 +1966,14 @@ VLC const TMNMVtab2[] = {
 };
 
 short const dc_threshold[] = {
-	21514, 26984,  8307, 28531, 29798, 24951, 25970, 26912,
-	 8307, 25956, 26994, 25974,  8292, 29286, 28015, 29728,
-	25960, 18208, 21838, 18208, 19536, 22560, 26998,  8260,
-	28515, 25956,  8291, 25640, 30309, 27749, 11817, 22794,
-	30063,  8306, 28531, 29798, 24951, 25970, 25632, 29545,
-	29300, 25193, 29813, 29295, 26656, 29537, 29728,  8303,
-	26983, 25974, 24864, 25443, 29541,  8307, 28532, 26912,
-	29556, 29472, 30063, 25458,  8293, 28515, 25956,  2606
+	26708, 29545, 29472, 26223, 30580, 29281,  8293, 29545,
+	25632, 29285, 30313, 25701, 26144, 28530,  8301, 26740,
+	 8293, 20039,  8277, 20551,  8268, 30296, 17513, 25376,
+	25711, 25445, 10272, 11825, 11825, 10544,  2606, 28505,
+	29301, 29472, 26223, 30580, 29281,  8293, 26980, 29811,
+	26994, 30050, 28532,  8306, 24936,  8307, 28532, 26400,
+	30313,  8293, 25441, 25955, 29555, 29728,  8303, 29801,
+	 8307, 28531, 29301, 25955, 25376, 25711, 11877,    10
 };
 
 VLC const dc_lum_tab[] = {
